@@ -70,7 +70,10 @@ static void update_progressbar (gpointer data, gpointer user_data)
 
 static gboolean timer_func (gpointer data)
 {
+	gdk_threads_enter ();
 	g_list_foreach (progressbars, update_progressbar, NULL);
+	gdk_threads_leave ();
+	
 	if (--pboffset < 0) pboffset = 9;
 	return (g_list_first(progressbars) != NULL);
 }
@@ -261,8 +264,8 @@ draw_flat_box (DRAW_ARGS)
 	if (detail && 	
 	    clearlooks_style->listviewitemstyle == 1 && 
 	    state_type == GTK_STATE_SELECTED && (
-	    !strncmp ("cell_even", detail, strlen ("cell_even")) ||
-	    !strncmp ("cell_odd", detail, strlen ("cell_odd"))))
+	    !strncmp ("cell_even", detail, 9) ||
+	    !strncmp ("cell_odd", detail, 8)))
 	{
 		GdkGC    *gc;
 		GdkColor  lower_color;
@@ -364,7 +367,7 @@ draw_shadow (DRAW_ARGS)
 			gdk_gc_set_clip_rectangle (clearlooks_style->shade_gc[0], NULL);
 		}
 	}
-	else if (detail && !strcmp (detail, "entry"))
+	else if (DETAIL ("entry"))
 	{
 		if ( widget->parent && (GTK_IS_COMBO_BOX_ENTRY (widget->parent) ||
 		                        GTK_IS_SPIN_BUTTON(widget) ||
@@ -834,7 +837,7 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 	gdk_gc_set_clip_rectangle (light_gc, &dest);
 	gdk_gc_set_clip_rectangle (dark_gc, &dest);
 	
-	n_lines = (!strcmp (detail, "paned")) ? 21 : 11;
+	n_lines = DETAIL ("paned") ? 21 : 11;
 
 	if (orientation == GTK_ORIENTATION_VERTICAL)
 	{	
@@ -911,10 +914,9 @@ draw_box (DRAW_ARGS)
 		cl_draw_treeview_header (style, window, state_type, shadow_type,
 		                         area, widget, detail, x, y, width, height);
 	}
-	else if (detail && (!strcmp (detail, "button") ||
-		                !strcmp (detail, "buttondefault")))
+	else if (DETAIL ("button") || DETAIL ("buttondefault"))
 	{
-		if (GTK_IS_COMBO_BOX_ENTRY(widget->parent) || GTK_IS_COMBO(widget->parent))
+		if (widget && (GTK_IS_COMBO_BOX_ENTRY(widget->parent) || GTK_IS_COMBO(widget->parent)))
 		{
 			cl_draw_combobox_button (style, window, state_type, shadow_type,
 			                         area, widget,
@@ -926,27 +928,22 @@ draw_box (DRAW_ARGS)
 		                    detail, x, y, width, height);
 		}
 	}
-	else if (detail && (
-	         !strcmp (detail, "spinbutton_up") ||
-	         !strcmp (detail, "spinbutton_down") ||
-	         !strcmp (detail, "spinbutton")))
+	else if (DETAIL ("spinbutton_up") ||
+	         DETAIL ("spinbutton_down") ||
+	         DETAIL ("spinbutton"))
 	{		
 		cl_draw_spinbutton (style, window, state_type, shadow_type, area,
 		                    widget, detail, x, y, width, height);
 	}
-	else if (detail && (
-		     !strcmp (detail, "hscale") || !strcmp (detail, "vscale")))
+	else if (DETAIL ("hscale") || DETAIL ("vscale"))
 	{
 		cl_rectangle_set_button (&r, style, state_type,
 		                widget && GTK_WIDGET_HAS_DEFAULT (widget), widget && GTK_WIDGET_HAS_FOCUS (widget),
 		                CL_CORNER_ROUND, CL_CORNER_ROUND,
 		                CL_CORNER_ROUND, CL_CORNER_ROUND);
 
-		if (!strcmp (detail, "hscale") || !strcmp (detail, "vscale"))
-		{
-			r.fill_gradient.to = &clearlooks_style->cl_shade[2];
-			r.bottomright = clearlooks_style->shade_gc[2];
-		}
+		r.fill_gradient.to = &clearlooks_style->cl_shade[2];
+		r.bottomright = clearlooks_style->shade_gc[2];
 		
 		cl_set_corner_sharpness (detail, widget, &r);
 	
@@ -1075,9 +1072,8 @@ draw_box (DRAW_ARGS)
 		cl_draw_rectangle (window, widget, style, x, y, width, height, &r);
 		cl_rectangle_reset_clip_rectangle (&r);
 	}
-	else if (detail && (!strcmp (detail, "vscrollbar") ||
-	                    !strcmp (detail, "hscrollbar") ||
-			    !strcmp (detail, "stepper")))
+	else if (DETAIL ("vscrollbar") || DETAIL ("hscrollbar") ||
+	         DETAIL ("stepper"))
 	{
 		ClScrollButtonType button_type = CL_SCROLLBUTTON_OTHER;
 		gboolean horizontal = TRUE;
@@ -1099,6 +1095,10 @@ draw_box (DRAW_ARGS)
 				button_type = CL_SCROLLBUTTON_END;
 		}
 
+		/* GTK+ 2.10 draws disabled stepper buttons. I don't like that. Looks weird. */
+		if (state_type == GTK_STATE_INSENSITIVE)
+			state_type = GTK_STATE_NORMAL;
+
 		cl_rectangle_set_button (&r, style, state_type, FALSE, FALSE, 0,0,0,0);
 		
 		cl_rectangle_set_gradient (&r.fill_gradient, NULL, NULL);
@@ -1172,7 +1172,7 @@ draw_box (DRAW_ARGS)
 			}
 		}
 			
-		cl_rectangle_set_button (&r, style, state_type, FALSE, GTK_WIDGET_HAS_FOCUS (widget),
+		cl_rectangle_set_button (&r, style, state_type, FALSE, widget && GTK_WIDGET_HAS_FOCUS (widget),
 		                        CL_CORNER_NONE, CL_CORNER_NONE,
 		                        CL_CORNER_NONE, CL_CORNER_NONE);
 		
@@ -1190,7 +1190,7 @@ draw_box (DRAW_ARGS)
 		cl_draw_shadow (window, widget, style, x, y, width, height, &r);
 		cl_rectangle_reset_clip_rectangle (&r);	
 	}
-	else if (detail && !strcmp (detail, "optionmenu")) /* supporting deprecated */
+	else if (DETAIL ("optionmenu")) /* supporting deprecated */
 	{
 		cl_draw_optionmenu(style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 	}
@@ -1243,7 +1243,7 @@ draw_box (DRAW_ARGS)
 		cl_draw_rectangle (window, widget, style, x, y, width, height, &r);
 		cl_rectangle_reset_clip_rectangle (&r);
 	}
-	else if (DETAIL ("menu") && widget->parent &&
+	else if (DETAIL ("menu") && widget && widget->parent &&
 	         GDK_IS_WINDOW (widget->parent->window))
 	{	
 		cl_rectangle_set_corners (&r, CL_CORNER_NONE, CL_CORNER_NONE,
@@ -1449,7 +1449,7 @@ draw_slider (DRAW_ARGS, GtkOrientation orientation)
 		(orientation == GTK_ORIENTATION_HORIZONTAL && width < 20))
 		return;
 	
-	if (detail && strcmp ("slider", detail) == 0)
+	if (DETAIL ("slider"))
 	{
 		if (area)
 		{
@@ -1484,7 +1484,7 @@ draw_slider (DRAW_ARGS, GtkOrientation orientation)
 			gdk_gc_set_clip_rectangle (white_gc, NULL);
 		}
 	}
-	else if (detail && (strcmp ("hscale", detail) == 0 || strcmp ("vscale", detail) == 0))
+	else if (DETAIL ("hscale") || DETAIL ("vscale"))
 	{
 		if (area)
 		{
@@ -1796,7 +1796,7 @@ draw_hline (GtkStyle     *style,
 	if (area)
 		gdk_gc_set_clip_rectangle (clearlooks_style->shade_gc[2], area);
 	
-	if (detail && !strcmp (detail, "label"))
+	if (DETAIL ("label"))
 	{
 		if (state_type == GTK_STATE_INSENSITIVE)
 			gdk_draw_line (window, style->light_gc[state_type], x1 + 1, y + 1, x2 + 1, y + 1);
@@ -1893,7 +1893,7 @@ draw_focus (GtkStyle      *style,
 	                            GDK_CAP_BUTT, GDK_JOIN_MITER);
 	
 	
-	if (detail && !strcmp (detail, "add-mode"))
+	if (DETAIL ("add-mode"))
 	{
 		if (free_dash_list)
 			g_free (dash_list);
